import io
import pathlib
from typing import List, Dict
import nbformat

def read_markdown(path: pathlib.Path) -> str:
    with open(path, 'r', encoding='utf8') as f:
        text = f.read()
    return text

def read_jupyter(path: pathlib.Path):
    with io.open(path, 'r', encoding='utf-8') as f:
        nb = nbformat.read(f,nbformat.NO_CONVERT)
        # nb = current.read(f, 'json')
    return nb

supported_formats = {
    '.md': read_markdown,
    '.ipynb': read_jupyter
}

def read_file(path: pathlib.Path) -> str:

    if path.suffix not in supported_formats.keys():
        print(f"{path.suffix} is not supported yet.")
        return Exception

    return supported_formats[path.suffix](path)

def read_path_list(path_list: List[pathlib.Path]) -> Dict[pathlib.Path,str]:

    content_dict: Dict[pathlib.Path, str] = {}

    for path in path_list:
        content_dict[path] = read_file(path)

    return content_dict



import nbformat

def count_words_in_jupyter(jupyter_json_content: nbformat.NotebookNode, return_type:str = 'markdown') -> int:
    word_count: int = 0
    for cell in jupyter_json_content.cells:
        if cell.cell_type != return_type:
            continue
        word_count += len(cell['source'].replace('#','').lstrip().split(' '))

    return word_count


import pathlib
from typing import List


def locate_files(path: str,file_type:str) -> List[pathlib.Path]:
    top_path: pathlib.Path = pathlib.Path(path)

    if "." not in file_type:
        print("File types must include a period eg, '.md'")

    located_files: List[pathlib.Path] = []

    for found_file_path in top_path.glob(f'**/*{file_type}'):
        if file_type != '.ipynb':
            located_files.append(found_file_path)
        else:
            if 'checkpoint' not in found_file_path.name:
                located_files.append(found_file_path)

    return located_files



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wordsum', 'wordsum._file_types', 'wordsum._io', 'wordsum._util']

package_data = \
{'': ['*']}

install_requires = \
['black>=19.10b0,<20.0', 'mypy>=0.770,<0.771', 'nbformat>=5.0.4,<6.0.0']

setup_kwargs = {
    'name': 'wordsum',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'JackMcKew',
    'author_email': 'jackmckew2@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import re

def count_words_in_markdown(markdown_text_content: str) -> int:

    # Comments
    markdown_text_content = re.sub(r'<!--(.*?)-->', '', markdown_text_content, flags=re.MULTILINE)
    # Tabs to spaces
    markdown_text_content = markdown_text_content.replace('\t', '    ')
    # More than 1 space to 4 spaces
    markdown_text_content = re.sub(r'[ ]{2,}', '    ', markdown_text_content)
    # Footnotes
    markdown_text_content = re.sub(r'^\[[^]]*\][^(].*', '', markdown_text_content, flags=re.MULTILINE)
    # Indented blocks of code
    markdown_text_content = re.sub(r'^( {4,}[^-*]).*', '', markdown_text_content, flags=re.MULTILINE)
    # Custom header IDs
    markdown_text_content = re.sub(r'{#.*}', '', markdown_text_content)
    # Replace newlines with spaces for uniform handling
    markdown_text_content = markdown_text_content.replace('\n', ' ')
    # Remove images
    markdown_text_content = re.sub(r'!\[[^\]]*\]\([^)]*\)', '', markdown_text_content)
    # Remove HTML tags
    markdown_text_content = re.sub(r'</?[^>]*>', '', markdown_text_content)
    # Remove special characters
    markdown_text_content = re.sub(r'[#*`~\-–^=<>+|/:]', '', markdown_text_content)
    # Remove footnote references
    markdown_text_content = re.sub(r'\[[0-9]*\]', '', markdown_text_content)
    # Remove enumerations
    markdown_text_content = re.sub(r'[0-9#]*\.', '', markdown_text_content)

    return len(markdown_text_content.split())



from wordsum._util.file_locate import locate_files
from wordsum._io.read_files import read_path_list
from wordsum._file_types.markdown import count_words_in_markdown
from wordsum._file_types.jupyter import count_words_in_jupyter

from typing import List

def count_words(path: str,file_types: List[str]) -> int:
    total_word_count: int = 0
    for file_type in file_types:
        if "." not in file_type:
            print(f"File Type: '{file_type}' must be formatted with a period (eg, '.md')")
            continue
        file_list = locate_files(path,file_type)
        file_content = read_path_list(file_list)
        for single_path, content in file_content.items():
            if file_type == '.md':
                total_word_count += count_words_in_markdown(content)
            elif file_type == '.ipynb':
                total_word_count += count_words_in_jupyter(content)

    return total_word_count

def list_supported_formats() -> List[str]:
    from wordsum._io.read_files import supported_formats
    print(f"wordsum supports {list(supported_formats.keys())}")
    return list(supported_formats.keys())


